#######################################################################
# Moral et al. - Location-scale mixed models and goodness-of-fit
#                asessment applied to insect ecology
#######################################################################

#######################################################################
# Loading data and packages
require(latticeExtra)
require(survival)
require(hnp)
source("helper_functions.R")

spod <- read.table("supplementarydata.txt", header = TRUE, row.names = 1)
spod <- transform(spod, block=as.factor(block),
                        attack=as.factor(attack),
                        consumption=as.factor(consumption),
                        consumption.2=as.factor(consumption.2))
spod$day <- gl(3, 44)
spod$block.lin <- rep(1:11, each=4)

#######################################################################
# Exploratory analysis

## Kaplan-Meier estimator
km.s <- survfit(Surv(log(time), cens) ~ species + gender, data=spod)
plot(km.s, col=c(2,2,1,1), lty=c(1,2), xlab="log(time)", ylab="S(y)")
legend(0, .25, c("Stinkbug","Earwig"), col=c(1,2), lwd=2, lty=1, 
       bty="n", title="Species", inset=.01)
legend(0, .45, c("Female","Male"), col=1, lwd=1, lty=c(1,2), bty="n", 
       title="Gender", inset=.01)

#######################################################################
# Weibull model

## Maximal linear predictor with block
X1b.w1 <- model.matrix(~ block + species * gender + relevel(consumption, "2"), 
                       data=spod)
X1b.w2 <- model.matrix(~ species * gender + relevel(consumption, "2"), 
                       data=spod)
fit1b.w <- optim(rep(0, 44), loglik.w, y=log(spod$time), cens=spod$cens, 
                 X1=X1b.w1, X2=X1b.w2, method="L-BFGS-B", hessian=T)

## Maximal linear predictor without block
X1.w1 <- model.matrix(~ species * gender + relevel(consumption, "2"), 
                      data=spod)
X1.w2 <- model.matrix(~ species * gender + relevel(consumption, "2"), 
                      data=spod)
fit1.w <- optim(rep(0, 12), loglik.w, y=log(spod$time), cens=spod$cens, 
                X1=X1.w1, X2=X1.w2, method="BFGS", hessian=T)
summary.ls(fit1.w, names=c("mu: Intercept","mu: Species - Stinkbug",
                           "mu: Gender - Male","mu: Consumption - Non-parasitised",
                           "mu: Consumption - Parasitised",
                           "mu: Species x Gender interaction",
                           "sigma: Intercept","sigma: Species - Stinkbug",
                           "sigma: Gender - Male",
                           "sigma: Consumption - Non-parasitised",
                           "sigma: Consumption - Parasitised",
                           "sigma: Species x Gender interaction"))

### Likelihood-ratio test
lrt(fit1b.w, fit1.w)

## Backward selection

### Removing interaction terms
X2.w1 <- model.matrix(~ species + gender + relevel(consumption, "2"), 
                      data=spod)
X2.w2 <- model.matrix(~ species + gender + relevel(consumption, "2"), 
                      data=spod)
fit2.w <- optim(rep(0, 10), loglik.w, y=log(spod$time), cens=spod$cens, 
                X1=X2.w1, X2=X2.w2, method="BFGS", hessian=T)
summary.ls(fit2.w, names=c("mu: Intercept","mu: Species - Stinkbug",
                           "mu: Gender - Male",
                           "mu: Consumption - Non-parasitised",
                           "mu: Consumption - Parasitised",
                           "sigma: Intercept","sigma: Species - Stinkbug",
                           "sigma: Gender - Male",
                           "sigma: Consumption - Non-parasitised",
                           "sigma: Consumption - Parasitised"))
#### Likelihood-ratio test
lrt(fit1.w, fit2.w)

### Removing gender effect
X3.w1 <- model.matrix(~ species + relevel(consumption, "2"), data=spod)
X3.w2 <- model.matrix(~ species + relevel(consumption, "2"), data=spod)
fit3.w <- optim(rep(0, 8), loglik.w, y=log(spod$time), cens=spod$cens, 
                X1=X3.w1, X2=X3.w2, method="BFGS", hessian=T)
summary.ls(fit3.w, names=c("mu: Intercept","mu: Species - Stinkbug",
                           "mu: Consumption - Non-parasitised",
                           "mu: Consumption - Parasitised",
                           "sigma: Intercept","sigma: Species - Stinkbug",
                           "sigma: Consumption - Non-parasitised",
                           "sigma: Consumption - Parasitised"))
#### Likelihood-ratio test
lrt(fit2.w, fit3.w)

### Combining levels of consumption
X4.w1 <- model.matrix(~ species + relevel(consumption.2, "2"), data=spod)
X4.w2 <- model.matrix(~ species + relevel(consumption.2, "2"), data=spod)
fit4.w <- optim(rep(0, 6), loglik.w, y=log(spod$time), cens=spod$cens, 
                X1=X4.w1, X2=X4.w2, method="BFGS", hessian=T)
summary.ls(fit4.w, names=c("mu: Intercept","mu: Species - Stinkbug",
                           "mu: Consumption - Yes",
                           "sigma: Intercept","sigma: Species - Stinkbug",
                           "sigma: Consumption - Yes"))
#### Likelihood-ratio test
lrt(fit3.w, fit4.w)

### Including species x consumption2 interaction
X5.w1 <- model.matrix(~ species * relevel(consumption.2, "2"), data=spod)
X5.w2 <- model.matrix(~ species * relevel(consumption.2, "2"), data=spod)
fit5.w <- optim(rep(0, 8), loglik.w, y=log(spod$time), cens=spod$cens, 
                X1=X5.w1, X2=X5.w2, method="BFGS", hessian=T)
summary.ls(fit5.w, names=c("mu: Intercept","mu: Species - Stinkbug",
                           "mu: Consumption - Yes",
                           "mu: Species x Consumption interaction",
                           "sigma: Intercept","sigma: Species - Stinkbug",
                           "sigma: Consumption - Yes",
                           "sigma: Species x Consumption interaction"))
#### Likelihood-ratio test
lrt(fit4.w, fit5.w)

### Removing consumption.2 effect from sigma
X6.w1 <- model.matrix(~ species + relevel(consumption.2, "2"), data=spod)
X6.w2 <- model.matrix(~ species, data=spod)
fit6.w <- optim(rep(0, 5), loglik.w, y=log(spod$time), cens=spod$cens, 
                X1=X6.w1, X2=X6.w2, method="BFGS", hessian=T)
summary.ls(fit6.w, names=c("mu: Intercept","mu: Species - Stinkbug",
                           "mu: Consumption - Yes",
                           "sigma: Intercept","sigma: Species - Stinkbug"))
#### Likelihood-ratio test
lrt(fit4.w, fit6.w)

##### final model: mu = species + consumption2
#####      log(sigma) = species + consumption2
##### global tests
lrt(fit1b.w, fit4.w) # including block
lrt(fit1.w, fit4.w) # without block

## Fitted survival curves
### Species x Gender
fitS.w <- optim(rep(0, 4), loglik.w, y=log(spod$time), cens=spod$cens, 
                X1=model.matrix(~ species, spod), 
                X2=model.matrix(~ species, spod), method="BFGS", 
                hessian=T)
plot(km.s, col=c(2,2,1,1), lty=c(1,2), xlab="log(time)", 
     ylab="S(y)", main="(a)")
legend(0, .25, c("Stinkbug","Earwig"), col=c(1,2), lwd=2, lty=1, 
       bty="n", title="Species", inset=.01)
legend(0, .45, c("Female","Male"), col=1, lwd=1, lty=c(1,2), 
       bty="n", title="Gender", inset=.01)
curve(Sw(x, fitS.w$par, rbind(c(1,0)), rbind(c(1,0))), 
      add=T, lwd=2, col=2)
curve(Sw(x, fitS.w$par, rbind(c(1,1)), rbind(c(1,1))), 
      add=T, lwd=2, col=1)

### Species x Consumption
km.s2 <- survfit(Surv(log(time), cens) ~ species + consumption.2, 
                 data=spod)
plot(km.s2, col=c(2,2,1,1), lty=c(1,2), xlab="log(time)", 
     ylab="S(y)", main="(b)")
legend(0, .25, c("Stinkbug","Earwig"), col=c(1,2), lwd=2, 
       lty=1, bty="n", title="Species", inset=.01)
legend(0, .45, c("Yes","No"), col=1, lwd=1, lty=c(1,2), 
       bty="n", title="  Consumption", inset=.01)
curve(Sw(x, fit4.w$par, rbind(c(1,0,1)), rbind(c(1,0,1))), 
      add=T, lwd=2, col=2)
curve(Sw(x, fit4.w$par, rbind(c(1,0,0)), rbind(c(1,0,0))), 
      add=T, lwd=2, col=2, lty=2)
curve(Sw(x, fit4.w$par, rbind(c(1,1,1)), rbind(c(1,1,1))), 
      add=T, lwd=2, col=1)
curve(Sw(x, fit4.w$par, rbind(c(1,1,0)), rbind(c(1,1,0))), 
      add=T, lwd=2, col=1, lty=2)

## Half-normal plot
X.w1 <- model.matrix(~ species * gender + consumption.2, spod)
X.w2 <- model.matrix(~ species * gender + consumption.2, spod)
model.w <- fit.w(log(spod$time), X1=X.w1, X2=X.w2, method="Nelder-Mead", resid.type=deviance.w)
set.seed(2019)
hnp.w <- hnp(model.w, newclass=T, diagfun=dfun.w, simfun=sfun.w, 
             fitfun=ffun.w, print=T, verb=T)

#######################################################################
# Exponentiated-Weibull model

## Maximal linear predictor with block
X1b.ew1 <- model.matrix(~ block + species * gender + relevel(consumption, "2"), 
                       data=spod)
X1b.ew2 <- model.matrix(~ species * gender + relevel(consumption, "2"), 
                       data=spod)
fit1b.ew <- optim(rep(.5, 45), loglik.ew, y=log(spod$time), cens=spod$cens, 
                 X1=X1b.ew1, X2=X1b.ew2, method="BFGS", hessian=T)

## Maximal linear predictor without block
X1.ew1 <- model.matrix(~ species * gender + relevel(consumption, "2"), 
                       data=spod)
X1.ew2 <- model.matrix(~ species * gender + relevel(consumption, "2"), 
                       data=spod)
fit1.ew <- optim(rep(.5, 13), loglik.ew, y=log(spod$time), cens=spod$cens, 
                 X1=X1.ew1, X2=X1.ew2, method="BFGS", hessian=T)
summary.ls(fit1.ew, names=c("a","mu: Intercept","mu: Species - Stinkbug",
                            "mu: Gender - Male",
                            "mu: Consumption - Non-parasitised",
                            "mu: Consumption - Parasitised",
                            "mu: Species x Gender interaction",
                            "sigma: Intercept","sigma: Species - Stinkbug",
                            "sigma: Gender - Male",
                            "sigma: Consumption - Non-parasitised",
                            "sigma: Consumption - Parasitised",
                            "sigma: Species x Gender interaction"))

### Likelihood-ratio test
lrt(fit1b.ew, fit1.ew)

## Backward selection

### Removing interaction terms
X2.ew1 <- model.matrix(~ species + gender + relevel(consumption, "2"), 
                       data=spod)
X2.ew2 <- model.matrix(~ species + gender + relevel(consumption, "2"), 
                       data=spod)
fit2.ew <- optim(rep(.5, 11), loglik.ew, y=log(spod$time), cens=spod$cens, 
                 X1=X2.ew1, X2=X2.ew2, method="BFGS", hessian=T)
summary.ls(fit2.ew, names=c("a","mu: Intercept","mu: Species - Stinkbug",
                            "mu: Gender - Male",
                            "mu: Consumption - Non-parasitised",
                            "mu: Consumption - Parasitised",
                            "sigma: Intercept",
                            "sigma: Species - Stinkbug",
                            "sigma: Gender - Male",
                            "sigma: Consumption - Non-parasitised",
                            "sigma: Consumption - Parasitised"))
#### Likelihood-ratio test
lrt(fit1.ew, fit2.ew)

### Removing gender effect
X3.ew1 <- model.matrix(~ species + relevel(consumption, "2"), data=spod)
X3.ew2 <- model.matrix(~ species + relevel(consumption, "2"), data=spod)
fit3.ew <- optim(rep(1, 9), loglik.ew, y=log(spod$time), cens=spod$cens, 
                 X1=X3.ew1, X2=X3.ew2, method="BFGS", hessian=T)
summary.ls(fit3.ew, names=c("a","mu: Intercept","mu: Species - Stinkbug",
                            "mu: Consumption - Non-parasitised",
                            "mu: Consumption - Parasitised",
                            "sigma: Intercept","sigma: Species - Stinkbug",
                            "sigma: Consumption - Non-parasitised",
                            "sigma: Consumption - Parasitised"))
#### Likelihood-ratio test
lrt(fit2.ew, fit3.ew)

### Leaving only intercept for mu
X4.ew1 <- model.matrix(~ 1, data=spod)
X4.ew2 <- model.matrix(~ species + relevel(consumption, "2"), data=spod)
fit4.ew <- optim(rep(1, 6), loglik.ew, y=log(spod$time), cens=spod$cens, 
                 X1=X4.ew1, X2=X4.ew2, method="BFGS", hessian=T)
summary.ls(fit4.ew, names=c("a","mu: Intercept",
                           "sigma: Intercept","sigma: Species - Stinkbug",
                           "sigma: Consumption - Non-parasitised",
                           "sigma: Consumption - Parasitised"))
#### Likelihood-ratio test
lrt(fit3.ew, fit4.ew)

### Combining levels for consumption
X5.ew1 <- model.matrix(~ 1, data=spod)
X5.ew2 <- model.matrix(~ species + relevel(consumption.2, "2"), data=spod)
fit5.ew <- optim(rep(1, 5), loglik.ew, y=log(spod$time), cens=spod$cens, 
                 X1=X5.ew1, X2=X5.ew2, method="BFGS", hessian=T)
summary.ls(fit5.ew, names=c("a","mu: Intercept",
                            "sigma: Intercept","sigma: Species - Stinkbug",
                            "sigma: Consumption - Yes"))
#### Likelihood-ratio test
lrt(fit4.ew, fit5.ew)

### Including species x consumption2 interaction
X6.ew1 <- model.matrix(~ 1, data=spod)
X6.ew2 <- model.matrix(~ species * relevel(consumption.2, "2"), data=spod)
fit6.ew <- optim(rep(1, 6), loglik.ew, y=log(spod$time), cens=spod$cens, 
                 X1=X6.ew1, X2=X6.ew2, method="BFGS", hessian=T)
summary.ls(fit6.ew, names=c("a","mu: Intercept",
                           "sigma: Intercept","sigma: Species - Stinkbug",
                           "sigma: Consumption - Yes",
                           "sigma: Species x Consumption interaction"))
#### Likelihood-ratio test
lrt(fit5.ew, fit6.ew)

##### final model: mu = intercept
#####      log(sigma) = species + consumption2
##### global tests
lrt(fit1b.ew, fit5.ew) # including block
lrt(fit1.ew, fit5.ew) # without block

## Fitted survival curves
### Species x Gender
fitS.ew <- optim(rep(1, 4), loglik.ew, y=log(spod$time), cens=spod$cens, 
                 X1=model.matrix(~ 1, spod), 
                 X2=model.matrix(~ species, spod), method="BFGS", hessian=T)
plot(km.s, col=c(2,2,1,1), lty=c(1,2), xlab="log(time)", ylab="S(y)", main="(a)")
legend(0, .25, c("Stinkbug","Earwig"), col=c(1,2), lwd=2, lty=1, 
       bty="n", title="Species", inset=.01)
legend(0, .45, c("Female","Male"), col=1, lwd=1, lty=c(1,2),
       bty="n", title="Gender", inset=.01)
curve(Sew(x, fitS.ew$par, rbind(c(1)), rbind(c(1,0))), add=T, lwd=2, col=2)
curve(Sew(x, fitS.ew$par, rbind(c(1)), rbind(c(1,1))), add=T, lwd=2, col=1)

### Species x Consumption
km.s2 <- survfit(Surv(log(time), cens) ~ species + consumption.2, data=spod)
plot(km.s2, col=c(2,2,1,1), lty=c(1,2), xlab="log(time)", ylab="S(y)", main="(b)")
legend(0, .25, c("Stinkbug","Earwig"), col=c(1,2), 
       lwd=2, lty=1, bty="n", title="Species", inset=.01)
legend(0, .45, c("Yes","No"), col=1, lwd=1, lty=c(1,2), 
       bty="n", title="  Consumption", inset=.01)
curve(Sew(x, fit5.ew$par, rbind(c(1)), rbind(c(1,0,1))), 
      add=T, lwd=2, col=2)
curve(Sew(x, fit5.ew$par, rbind(c(1)), rbind(c(1,0,0))), 
      add=T, lwd=2, col=2, lty=2)
curve(Sew(x, fit5.ew$par, rbind(c(1)), rbind(c(1,1,1))), 
      add=T, lwd=2, col=1)
curve(Sew(x, fit5.ew$par, rbind(c(1)), rbind(c(1,1,0))), 
      add=T, lwd=2, col=1, lty=2)

## Half-normal plot
X.ew1 <- model.matrix(~ species * gender + consumption.2, spod)
X.ew2 <- model.matrix(~ species * gender + consumption.2, spod)
model.ew <- fit.ew(log(spod$time), X1=X.ew1, X2=X.ew2, resid.type=deviance.ew)
set.seed(2019)
hnp.ew <- hnp(model.ew, newclass=T, diagfun=dfun.ew, simfun=sfun.ew, 
              fitfun=ffun.ew, print=T, verb=T)

#########################################################################
# Exponentiated-Weibull mixed model
X1.ew.mix <- model.matrix(~ 1, data=spod)
X2.ew.mix <- model.matrix(~ species + relevel(consumption.2, "2"), data=spod)
fit.ew.mix <- optim(c(fit5.ew$par, 0), loglik.ew.mix, y=log(spod$time), cens=spod$cens, X1=X1.ew.mix, X2=X2.ew.mix, method="BFGS", hessian=T, type="integrate", limit=10)
fit.ew.mix2 <- fit.ew.mix
fit.ew.mix2$par <- fit.ew.mix2$par[-6]
fit.ew.mix2$hessian <- fit.ew.mix2$hessian[1:5,1:5]
summary.ls(fit.ew.mix2, names=c("a","mu: Intercept","sigma: Intercept","sigma: Species - Earwig","sigma: Consumption - Yes"))
rbind(fit.ew.mix$par, c(fit5.ew$par, NA))

lrt(fit.ew.mix, fit5.ew)[3]/2

## prediction of random effects
ranef.fitewmix <- ranef.ew.mix(theta=fit.ew.mix$par, y=log(spod$time), cens=spod$cens, X1=X1.ew.mix, X2=X2.ew.mix, limit=4, type="integrate")
var.ranef.fitewmix <- ranef.ew.mix(theta=fit.ew.mix$par, y=log(spod$time), cens=spod$cens, X1=X1.ew.mix, X2=X2.ew.mix, limit=4, type="integrate", squared=T) - ranef.fitewmix^2
incremento <- 1.96 * sqrt(var.ranef.fitewmix)
plot(ranef.fitewmix, 1:3, xlim=c(-.04,.04), yaxt="n", ylim=c(-1,5), xlab=expression(hat(b[i])), ylab="Day")
axis(2, 1:3, 1:3, las=2)
abline(v=0, lty=2, col="gray")
arrows(ranef.fitewmix + incremento, seq_along(ranef.fitewmix),
       ranef.fitewmix - incremento, seq_along(ranef.fitewmix),
       code=3, length=.05, angle=90)