spod <- read.table("moral_et_al_spodoptera.txt", h=T)
spod <- transform(spod, block=as.factor(block),
                  attack=as.factor(attack),
                  consumption=as.factor(consumption),
                  consumption.2=as.factor(consumption.2))
spod$day <- gl(3, 44)

int.approx <- function(f, type, n, limit, ...) {
  if(type=="GQ") {
    p <- ghq(n.points=n, modified=F)
    return(sum(p$weights * f(p$zeros, ...) * exp(p$zeros^2)))
  }
  if(type=="integrate") {
    integ <- integrate(f, lower=-limit, upper=limit, ...)
    return(integ$value)
  }
}

integrand <- Vectorize(function(bk, yi, a, beta1, beta2, sig.block, cens, X1, X2) {
  beta1[1] <- beta1[1] + bk
  mu <- X1%*%beta1
  sigma. <- exp(X2%*%beta2)
  f <- a/(sigma.)*(1-exp(-exp((yi-mu)/(sigma.))))^(a-1)*exp((yi-mu)/(sigma.))*exp(-exp((yi-mu)/(sigma.)))
  S <- 1-(1-exp(-exp((yi-mu)/(sigma.))))^a
  lk <- sum(cens*log(f) + (1-cens)*log(S)) + dnorm(bk, 0, sig.block, log=TRUE)
  if(is.nan(lk) | lk < -9999) return(0) else return(exp(lk))
}, "bk")

integral <- function(y, theta, cens, X1, X2, n, limit, type) {
  a <- exp(theta[1])
  beta1 <- theta[2:(2+ncol(X1)-1)]
  beta2 <- theta[(2+ncol(X1)):(2+ncol(X1)+ncol(X2)-1)]
  sig.block <- exp(theta[length(theta)])
  approx <- int.approx(integrand, yi=y, a=a, beta1=beta1, beta2=beta2, sig.block=sig.block, cens=cens, X1=X1, X2=X2, n=n, limit=limit, type=type)
  return(-log(approx))
}

loglik.ew.mix <- function(theta, y, cens, X1, X2, n, limit, type) {
  if(missing(n)) n <- 10
  if(missing(limit)) limit <- 30
  if(missing(type)) type <- "GQ"
  yi <- split(y, spod$day)
  censi <- split(cens, spod$day)
  X1i <- split(X1, spod$day)
  for(i in 1:3) X1i[[i]] <- matrix(X1i[[i]], ncol=ncol(X1))
  X2i <- split(X2, spod$day)
  for(i in 1:3) X2i[[i]] <- matrix(X2i[[i]], ncol=ncol(X2))
  liks <- NULL
  for(i in 1:3) {
    liks[i] <- integral(y=yi[[i]], theta=theta, cens=censi[[i]], X1=X1i[[i]], X2=X2i[[i]], n=n, limit=limit, type=type)
  }
  return(sum(liks))
}

###################################################################################
# Simulation study

## model matrices
X1.sim <- model.matrix(~ 1, data=spod)
X2.sim <- model.matrix(~ species + relevel(consumption.2, "2"), data=spod)
Xr.sim <- model.matrix(~ day - 1, data=spod)

## simulating data with day effect
sim.ew.day <- function(n, theta, X1, X2, sigma, X.random) {
  a <- exp(theta[1])
  ran.eff <- rnorm(ncol(X.random), 0, sigma)
  beta1 <- theta[2:(2+ncol(X1)-1)]
  beta2 <- theta[(2+ncol(X1)):(2+ncol(X1)+ncol(X2)-1)]
  mu <- X1%*%beta1 + X.random%*%ran.eff
  sigma. <- exp(X2%*%beta2)
  U <- runif(n)
  Y <- mu + sigma.*log(-log(1-U^(1/a)))
  Y <- ifelse(Y > log(3600), log(3600), Y)
  return(round(as.numeric(Y),6))
}

## Study 1: same sample size and estimated parameters from experiment; 
##          sigma=0, 1, 2, and 3

mc.study <- function(sigma, n=10, limit=30, type="GQ") {
  y.sim <- sim.ew.day(n=132, theta=c(2.17,2.78,1.10,0.47,-0.51), X1=X1.sim, X2=X2.sim, sigma=sigma, X.random=Xr.sim)
  cens.sim <- ifelse(y.sim >= log(3600), 0, 1)
  fit.ew.mix.sim <- try(optim(c(2.17,2.78,1.10,0.47,-0.51, ifelse(sigma==0, 0, log(sigma))), loglik.ew.mix, y=y.sim, cens=cens.sim, X1=X1.sim, X2=X2.sim, hessian=T, method="BFGS", n=n, limit=limit, type=type), silent=T)
  if(class(fit.ew.mix.sim)=="try-error") estimates <- rep(NA, 6) else estimates <- fit.ew.mix.sim$par
  #estimates[c(1,6)] <- exp(estimates[c(1,6)])
  #estimates[6] <- estimates[6]^2
  cat("Estimation procedure finished", "\n")
  return(estimates)
}

mc.sig0 <- replicate(1000, mc.study(0, type="integrate", limit=10))
s0 <- TRUE
save(s0, file="s0_done.rData")
mc.sig1 <- replicate(1000, mc.study(1, type="integrate", limit=10))
s1 <- TRUE
save(s1, file="s1_done.rData")
mc.sig3 <- replicate(1000, mc.study(3, type="integrate", limit=10))
s3 <- TRUE
save(s3, file="s3_done.rData")

### average, bias and MSE
theta.sim <- c(2.17,2.78,1.10,0.47,-0.51)

mean. <- function(x) mean(na.omit(x))
var. <- function(x) var(na.omit(x))

mc2.sig0 <- mc.sig0
mc2.sig0[6,] <- exp(mc2.sig0[6,])
mc2.sig1 <- mc.sig1
mc2.sig1[6,] <- exp(mc2.sig1[6,])
mc2.sig3 <- mc.sig3
mc2.sig3[6,] <- exp(mc2.sig3[6,])
mean0 <- apply(mc2.sig0, 1, mean.)
mean1 <- apply(mc2.sig1, 1, mean.)
mean3 <- apply(mc2.sig3, 1, mean.)

mse0 <- apply(mc.sig0, 1, var.) + (mean0 - c(theta.sim, 0))^2
mse1 <- apply(mc.sig1, 1, var.) + (mean1 - c(theta.sim, 1))^2
mse3 <- apply(mc.sig3, 1, var.) + (mean3 - c(theta.sim, 3))^2

study1 <- round(data.frame(row.names=c("a","mu","sig-intercept","sig-species","sig-cons2","sigma2"),
                           c(theta.sim, NA), mean0, mean1, mean3), 4)
study1.mse <- round(data.frame(row.names=c("a","mu","sig-intercept","sig-species","sig-cons2","sigma2"), mse0, mse1, mse3), 4)

write.csv(file="study1.csv", data.frame(study1, study1.mse))