spod <- read.table("moral_et_al_spodoptera.txt", h=T)
spod <- transform(spod, block=as.factor(block),
                  attack=as.factor(attack),
                  consumption=as.factor(consumption),
                  consumption.2=as.factor(consumption.2))
spod$day <- gl(3, 44)

int.approx <- function(f, type, n, limit, ...) {
  if(type=="GQ") {
    p <- ghq(n.points=n, modified=F)
    return(sum(p$weights * f(p$zeros, ...) * exp(p$zeros^2)))
  }
  if(type=="integrate") {
    integ <- integrate(f, lower=-limit, upper=limit, ...)
    return(integ$value)
  }
}

integrand <- Vectorize(function(bk, yi, a, beta1, beta2, sig.block, cens, X1, X2) {
  beta1[1] <- beta1[1] + bk
  mu <- X1%*%beta1
  sigma. <- exp(X2%*%beta2)
  f <- a/(sigma.)*(1-exp(-exp((yi-mu)/(sigma.))))^(a-1)*exp((yi-mu)/(sigma.))*exp(-exp((yi-mu)/(sigma.)))
  S <- 1-(1-exp(-exp((yi-mu)/(sigma.))))^a
  lk <- sum(cens*log(f) + (1-cens)*log(S)) + dnorm(bk, 0, sig.block, log=TRUE)
  if(is.nan(lk) | lk < -9999) return(0) else return(exp(lk))
}, "bk")

integral <- function(y, theta, cens, X1, X2, n, limit, type) {
  a <- exp(theta[1])
  beta1 <- theta[2:(2+ncol(X1)-1)]
  beta2 <- theta[(2+ncol(X1)):(2+ncol(X1)+ncol(X2)-1)]
  sig.block <- exp(theta[length(theta)])
  approx <- int.approx(integrand, yi=y, a=a, beta1=beta1, beta2=beta2, sig.block=sig.block, cens=cens, X1=X1, X2=X2, n=n, limit=limit, type=type)
  return(-log(approx))
}

loglik.ew.mix <- function(theta, y, cens, X1, X2, n, limit, type) {
  if(missing(n)) n <- 10
  if(missing(limit)) limit <- 30
  if(missing(type)) type <- "GQ"
  yi <- split(y, spod$day)
  censi <- split(cens, spod$day)
  X1i <- split(X1, spod$day)
  for(i in 1:3) X1i[[i]] <- matrix(X1i[[i]], ncol=ncol(X1))
  X2i <- split(X2, spod$day)
  for(i in 1:3) X2i[[i]] <- matrix(X2i[[i]], ncol=ncol(X2))
  liks <- NULL
  for(i in 1:3) {
    liks[i] <- integral(y=yi[[i]], theta=theta, cens=censi[[i]], X1=X1i[[i]], X2=X2i[[i]], n=n, limit=limit, type=type)
  }
  return(sum(liks))
}

###################################################################################
# Simulation study

## model matrices
X1.sim <- model.matrix(~ 1, data=spod)
X2.sim <- model.matrix(~ species + relevel(consumption.2, "2"), data=spod)
Xr.sim <- model.matrix(~ day - 1, data=spod)

## simulating data with day effect
sim.ew.day <- function(n, theta, X1, X2, sigma, X.random) {
  a <- exp(theta[1])
  ran.eff <- rnorm(ncol(X.random), 0, sigma)
  beta1 <- theta[2:(2+ncol(X1)-1)]
  beta2 <- theta[(2+ncol(X1)):(2+ncol(X1)+ncol(X2)-1)]
  mu <- X1%*%beta1 + X.random%*%ran.eff
  sigma. <- exp(X2%*%beta2)
  U <- runif(n)
  Y <- mu + sigma.*log(-log(1-U^(1/a)))
  Y <- ifelse(Y > log(3600), log(3600), Y)
  return(round(as.numeric(Y),6))
}

## Study 2: same estimated parameters from the experiment

loglik.ew.mix2 <- function(theta, y, cens, X1, X2, spl, n, limit, type) {
  n. <- length(unique(spl))
  yi <- split(y, spl)
  censi <- split(cens, spl)
  X1i <- split(X1, spl)
  for(i in 1:n.) X1i[[i]] <- matrix(X1i[[i]], ncol=ncol(X1))
  X2i <- split(X2, spl)
  for(i in 1:n.) X2i[[i]] <- matrix(X2i[[i]], ncol=ncol(X2))
  liks <- NULL
  for(i in 1:n.) {
    liks[i] <- integral(y=yi[[i]], theta=theta, cens=censi[[i]], X1=X1i[[i]], X2=X2i[[i]], n=n, limit=limit, type=type)
  }
  return(sum(liks))
}

mc.study2 <- function(sigma, size, censoring, n.ranef, n, limit, type) {
  spl. <- gl(n.ranef, size/n.ranef)
  X1 <- matrix(1, nrow=size)
  X2 <- as.matrix(data.frame(rep(1, size), c(0,1), c(0,0,1,1)))
  Xr <- model.matrix(~ spl. - 1)
  y.sim <- sim.ew.day(n=size, theta=c(2.17,2.78,1.10,0.47,-0.51), X1=X1, X2=X2, sigma=sigma, X.random=Xr)
  cens.sim <- 1 - rbinom(size, 1, censoring)
  fit.ew.mix.sim <- try(optim(c(2.17,2.78,1.10,0.47,-0.51, ifelse(sigma==0, 0, log(sigma))), loglik.ew.mix2, y=y.sim, cens=cens.sim, X1=X1, X2=X2, spl=spl., hessian=T, method="BFGS", n=n, limit=limit, type=type), silent=T)
  if(class(fit.ew.mix.sim)=="try-error") estimates <- rep(NA, 6) else estimates <- fit.ew.mix.sim$par
  cat("Estimation procedure finished", "\n")
  return(estimates)
}

sigmai <- c(0,1,3)
samplei <- c(36,132,480)
censori <- c(0,.15,.3)
n.ranefi <- c(4,12) # 12 random effects not meaningful for sample size = 36

study2 <- list()
marker <- 0
total <- length(sigmai) * length(samplei) * length(censori) * length(n.ranefi)
for(i in sigmai) {
  for(j in samplei) {
    for(k in censori) {
      for(l in n.ranefi) {
        marker <- marker + 1
        cat("Simulation study ", marker, "/", total, "\n", sep="")
        study2[[length(study2)+1]] <- replicate(1000, 
                                                mc.study2(sigma=i, size=j, censoring=k, n.ranef=l, 
                                                          type="integrate", limit=10))
        names(study2)[length(study2)] <- paste(i,j,k,l)
        save(l, file="l_loop.rData")
      }
      save(k, file="k_loop.rData")
    }
    save(j, file="j_loop.rData")
  }
  save(i, file="i_loop.rData")
}

study2. <- lapply(study2, function(x) {
  x[6,] <- exp(x[6,])
  return(x)
})

theta.sim <- c(2.17,2.78,1.10,0.47,-0.51)

mean. <- function(x) mean(na.omit(x))
var. <- function(x) var(na.omit(x))

mean.s2 <- as.data.frame(matrix(unlist(lapply(study2., function(x) apply(x, 1, mean.))), nrow=6, byrow=F))
names(mean.s2) <- names(study2.)

var.s2 <- as.data.frame(matrix(unlist(lapply(study2., function(x) apply(x, 1, var.))), nrow=6, byrow=F))
names(var.s2) <- names(study2.)

true.s2 <- as.data.frame(matrix(c(rep(c(theta.sim, 0), 18),
                                  rep(c(theta.sim, 1), 18),
                                  rep(c(theta.sim, 3), 18)), nrow=6, byrow=F))
mse.s2 <- var.s2 + (mean.s2 - true.s2)^2

study2.table <- round(data.frame(row.names=c("a","mu","sig-intercept","sig-species","sig-cons2","sigma2"),
                                 c(theta.sim, NA), mean.s2), 4)

write.csv(file="study2mean.csv", study2.table)
write.csv(file="study2mse.csv", mse.s2)