# Weibull model

## pdf
fw <- function(y, theta, X1, X2) {
  theta1 <- theta[1:ncol(X1)]
  theta2 <- theta[(ncol(X1)+1):length(theta)]
  mu <- X1%*%theta1
  sigma <- exp(X2%*%theta2)
  1/sigma * exp((y - mu)/sigma) * exp(-exp((y - mu)/sigma))
}

## survival function
Sw <- function(y, theta, X1, X2) {
  theta1 <- theta[1:ncol(X1)]
  theta2 <- theta[(ncol(X1)+1):length(theta)]
  mu <- X1%*%theta1
  sigma <- exp(X2%*%theta2)
  exp(-exp((y - mu)/sigma))
}

## log-likelihood function
loglik.w <- function(theta, y, cens, X1, X2) {
  theta1 <- theta[1:ncol(X1)]
  theta2 <- theta[(ncol(X1)+1):length(theta)]
  mu <- X1%*%theta1
  sigma <- exp(X2%*%theta2)
  llik <- cens * (-log(sigma) + (y - mu)/sigma) - exp((y - mu)/sigma)
  return(-sum(llik))
}

## summary for location scale models fitted using optim()
summary.ls <- function(obj, names) {
  pars <- obj$par
  ep <- sqrt(diag(solve(obj$hessian)))
  if(missing(names)) names <- c(1:length(pars))
  tv <- pars/ep
  pv <- round(pnorm(abs(tv), lower=F)*2,5)
  sig <- ifelse(pv<.05,"*",ifelse(pv<.1,".",""))
  ret <- data.frame(parameter=names, value=pars, std.error=ep, 
                    t.value=tv, p.value=pv, s=sig)
  return(ret)
}

## function for likelihood-ratio tests
lrt <- function(model1, model2) {
  d.f <- abs(length(model1$par) - length(model2$par))
  x2 <- 2 * abs(model1$value - model2$value)
  p <- 1 - pchisq(x2, d.f)
  return(data.frame("Chisq"=x2, "df"=d.f, "p-value"=p))
}

## Half-normal plot for Weibull model
### function to simulate a random sample
sim.w <- function(n, theta, X1, X2) {
  theta1 <- theta[1:ncol(X1)]
  theta2 <- theta[(ncol(X1)+1):length(theta)]
  mu <- X1%*%theta1
  sigma <- exp(X2%*%theta2)
  U <- runif(n, 0, 1)
  Y <- mu + sigma * log(-log(1 - U))
  return(Y)
}

### function to obtain Cox-Snell residuals
cox.snell.w <- function(y, theta, X1, X2) {
  theta1 <- theta[1:ncol(X1)]
  theta2 <- theta[(ncol(X1)+1):length(theta)]
  mu <- X1%*%theta1
  sigma <- exp(X2%*%theta2)
  # S <- exp(-exp((y - mu)/sigma))
  # logS <- -exp((y - mu)/sigma)
  return(exp((y - mu)/sigma))
}

### modified Cox-Snell residuals
mod.cox.snell.w <- function(y, delta, theta, X1, X2, ...) {
  cox.snell.w(y, theta, X1, X2, ...) + 1 - delta
}

### function to obtain Martingale residuals
martingale.w <- function(y, delta, theta, X1, X2, ...) {
  delta - cox.snell.w(y, theta, X1, X2, ...)
}

### function to obtain deviance residuals
deviance.w <- function(y, delta, theta, X1, X2, ...) {
  mi <- martingale.w(y, delta, theta, X1, X2, ...)
  sign(mi)*sqrt(-2*(mi+delta*log(delta-mi)))
}

### function to fit the Weibull model
fit.w <- function(resp, X1, X2, init=rep(0, ncol(X1)+ncol(X2)), method="BFGS", resid.type=deviance.w) {
  cens <- rep(1, 132)
  cens[resp>=log(3599)] <- 0
  resp[resp>=log(3599)] <- log(3600)
  opt <- optim(init, loglik.w, y=resp, cens=cens, X1=X1, X2=X2, 
               hessian=T, method=method)
  result <- list(pars=opt$par, res=resid.type(resp, delta=cens, 
                                              opt$par, X1, X2))
  return(result)
}

### function to extract deviance residuals from model fit
resid.w <- function(obj) as.numeric(obj$res)

### diagnostic extraction function (for hnp)
dfun.w <- function(obj) resid.w(obj)

### simulation function (for hnp)
sfun.w <- function(n, obj) {
  parms <- obj$pars
  sim.w(n, parms, X.w1, X.w2)
}

### model fitting function (for hnp)
ffun.w <- function(y.) fit.w(y., init=model.w$par, 
                             X1=X.w1, X2=X.w2, resid.type=deviance.w)

# Exponentiated-Weibull model
## pdf
few <- function(theta, y, X1, X2) {
  a <- exp(theta[1])
  beta1 <- theta[2:(2+ncol(X1)-1)]
  beta2 <- theta[(2+ncol(X1)):(2+ncol(X1)+ncol(X2)-1)]
  mu <- X1%*%beta1
  sigma <- exp(X2%*%beta2)
  f <- a/sigma*(1-exp(-exp((y-mu)/sigma)))^(a-1)*
    exp((y-mu)/sigma)*exp(-exp((y-mu)/sigma))
  return(f)
}

## survival function
Sew <- function(y, theta, X1, X2) {
  a <- exp(theta[1])
  beta1 <- theta[2:(2+ncol(X1)-1)]
  beta2 <- theta[(2+ncol(X1)):(2+ncol(X1)+ncol(X2)-1)]
  mu <- X1%*%beta1
  sigma <- exp(X2%*%beta2)
  sew <- 1-(1-exp(-exp((y-mu)/sigma)))^a
  return(sew)
}

## log-likelihood function
loglik.ew <- function(theta, y, cens, X1, X2) {
  a <- exp(theta[1])
  beta1 <- theta[2:(2+ncol(X1)-1)]
  beta2 <- theta[(2+ncol(X1)):(2+ncol(X1)+ncol(X2)-1)]
  mu <- X1%*%beta1
  sigma <- exp(X2%*%beta2)
  f <- a/sigma*(1-exp(-exp((y-mu)/sigma)))^(a-1)*
    exp((y-mu)/sigma)*exp(-exp((y-mu)/sigma))
  S <- 1-(1-exp(-exp((y-mu)/sigma)))^a
  lik <- cens*log(f) + (1-cens)*log(S)
  return(-sum(lik))
}

## summary for location scale models fitted using optim()
summary.ls <- function(obj, names) {
  pars <- obj$par
  ep <- sqrt(diag(solve(obj$hessian)))
  if(missing(names)) names <- c(1:length(pars))
  tv <- pars/ep
  pv <- round(pnorm(abs(tv), lower=F)*2,5)
  sig <- ifelse(pv<.05,"*",ifelse(pv<.1,".",""))
  ret <- data.frame(parameter=names, value=pars, std.error=ep, 
                    t.value=tv, p.value=pv, s=sig)
  return(ret)
}

## function for likelihood-ratio tests
lrt <- function(model1, model2) {
  d.f <- abs(length(model1$par) - length(model2$par))
  x2 <- 2 * abs(model1$value - model2$value)
  p <- 1 - pchisq(x2, d.f)
  return(data.frame("Chisq"=x2, "df"=d.f, "p-value"=p))
}

# Half-normal plot for exponentiated-Weibull model
### function to simulate a random sample
sim.ew <- function(n, theta, X1, X2) {
  a <- exp(theta[1])
  beta1 <- theta[2:(2+ncol(X1)-1)]
  beta2 <- theta[(2+ncol(X1)):(2+ncol(X1)+ncol(X2)-1)]
  mu <- X1%*%beta1
  sigma <- exp(X2%*%beta2)
  U <- runif(n)
  Y <- mu + sigma*log(-log(1-U^(1/a)))
  Y <- ifelse(Y > log(3600), log(3600), Y)
  return(round(as.numeric(Y),6))
}

### function to obtain Cox-Snell residuals
cox.snell.ew <- function(y, theta, X1, X2) {
  a <- exp(theta[1])
  beta1 <- theta[2:(2+ncol(X1)-1)]
  beta2 <- theta[(2+ncol(X1)):(2+ncol(X1)+ncol(X2)-1)]
  mu <- X1%*%beta1
  sigma <- exp(X2%*%beta2)
  S <- 1-(1-exp(-exp((y-mu)/sigma)))^a
  logS <- log(S)
  return(-logS)
}

### modified Cox-Snell residuals
mod.cox.snell.ew <- function(y, delta, theta, X1, X2, ...) {
  cox.snell.ew(y, theta, X1, X2, ...) + 1 - delta
}

### function to obtain Martingale residuals
martingale.ew <- function(y, delta, theta, X1, X2, ...) {
  delta - cox.snell.ew(y, theta, X1, X2, ...)
}

### function to obtain deviance residuals
deviance.ew <- function(y, delta, theta, X1, X2, ...) {
  mi <- martingale.ew(y, delta, theta, X1, X2, ...)
  sign(mi)*sqrt(-2*(mi+delta*log(delta-mi)))
}

### function to fit the exponentiated-Weibull model
fit.ew <- function(resp, init=rep(1, 1+ncol(X1)+ncol(X2)), 
                   X1, X2, method="BFGS", resid.type=deviance.ew) {
  cens <- rep(1, 132)
  cens[resp==8.188689] <- 0
  opt <- optim(init, loglik.ew, y=resp, cens=cens, X1=X1, X2=X2, 
               hessian=T, method=method)
  result <- list(pars=opt$par, res=resid.type(resp, cens, 
                                              opt$par, X1, X2))
  return(result)
}

### function to extract deviance residuals from model fit
resid.ew <- function(obj) as.numeric(obj$res)

### diagnostic extraction function (for hnp)
dfun.ew <- function(obj) resid.ew(obj)

### simulation function (for hnp)
sfun.ew <- function(n, obj) {
  parms <- obj$pars
  sim.ew(n, parms, X.ew1, X.ew2)
}

### model fitting function (for hnp)
ffun.ew <- function(y.) fit.ew(y., init=model.ew$par, 
                               X1=X.ew1, X2=X.ew2, resid.type=deviance.ew)

# Exponentiated-Weibull mixed model
require(glmmML)
require(statmod)

int.approx <- function(f, type, n, limit, ...) {
  if(type=="GQ") {
    p <- ghq(n.points=n, modified=F)
    return(sum(p$weights * f(p$zeros, ...) * exp(p$zeros^2)))
  }
  if(type=="integrate") {
    integ <- integrate(f, lower=-limit, upper=limit, ...)
    return(integ$value)
  }
}

integrand <- Vectorize(function(bk, yi, a, beta1, beta2, sig.block, cens, X1, X2) {
  beta1[1] <- beta1[1] + bk
  mu <- X1%*%beta1
  sigma. <- exp(X2%*%beta2)
  f <- a/(sigma.)*(1-exp(-exp((yi-mu)/(sigma.))))^(a-1)*exp((yi-mu)/(sigma.))*exp(-exp((yi-mu)/(sigma.)))
  S <- 1-(1-exp(-exp((yi-mu)/(sigma.))))^a
  lk <- sum(cens*log(f) + (1-cens)*log(S)) + dnorm(bk, 0, sig.block, log=TRUE)
  if(is.nan(lk) | lk < -9999) return(0) else return(exp(lk))
}, "bk")

integral <- function(y, theta, cens, X1, X2, n, limit, type) {
  a <- exp(theta[1])
  beta1 <- theta[2:(2+ncol(X1)-1)]
  beta2 <- theta[(2+ncol(X1)):(2+ncol(X1)+ncol(X2)-1)]
  sig.block <- exp(theta[length(theta)])
  approx <- int.approx(integrand, yi=y, a=a, beta1=beta1, beta2=beta2, sig.block=sig.block, cens=cens, X1=X1, X2=X2, n=n, limit=limit, type=type)
  return(-log(approx))
}

loglik.ew.mix <- function(theta, y, cens, X1, X2, n, limit, type) {
  if(missing(n)) n <- 10
  if(missing(limit)) limit <- 30
  if(missing(type)) type <- "GQ"
  yi <- split(y, spod$day)
  censi <- split(cens, spod$day)
  X1i <- split(X1, spod$day)
  for(i in 1:3) X1i[[i]] <- matrix(X1i[[i]], ncol=ncol(X1))
  X2i <- split(X2, spod$day)
  for(i in 1:3) X2i[[i]] <- matrix(X2i[[i]], ncol=ncol(X2))
  liks <- NULL
  for(i in 1:3) {
    liks[i] <- integral(y=yi[[i]], theta=theta, cens=censi[[i]], X1=X1i[[i]], X2=X2i[[i]], n=n, limit=limit, type=type)
  }
  return(sum(liks))
}

## prediction of random effects
int.helper <- Vectorize(function(bk, yi, a, beta1, beta2, sig.block, cens, X1, X2) {
  beta1[1] <- beta1[1] + bk
  mu <- X1%*%beta1
  sigma. <- exp(X2%*%beta2)
  f <- a/(sigma.)*(1-exp(-exp((yi-mu)/(sigma.))))^(a-1)*exp((yi-mu)/(sigma.))*exp(-exp((yi-mu)/(sigma.)))
  S <- 1-(1-exp(-exp((yi-mu)/(sigma.))))^a
  int.den <- sum(cens*log(f) + (1-cens)*log(S)) + dnorm(bk, 0, sig.block, log=TRUE)
  if(is.nan(int.den) | int.den < -9999) return(0) else return(exp(int.den))
}, "bk")

integrand.ranef <- Vectorize(function(bk, yi, a, beta1, beta2, sig.block, cens, X1, X2, squared) {
  beta1[1] <- beta1[1] + bk
  mu <- X1%*%beta1
  sigma. <- exp(X2%*%beta2)
  f <- a/(sigma.)*(1-exp(-exp((yi-mu)/(sigma.))))^(a-1)*exp((yi-mu)/(sigma.))*exp(-exp((yi-mu)/(sigma.)))
  S <- 1-(1-exp(-exp((yi-mu)/(sigma.))))^a
  numerator <- sum(cens*log(f) + (1-cens)*log(S)) + dnorm(bk, 0, sig.block, log=TRUE)
  denominator <- integrate(int.helper, lower=-10, upper=10, 
                           yi=yi, a=a, beta1=beta1, beta2=beta2, sig.block=sig.block, cens=cens, X1=X1, X2=X2)$value
  if(squared) bk <- bk^2
  linteg <- bk * exp(numerator - log(denominator))
  if(is.nan(linteg)) return(0) else return(linteg)
}, "bk")

integral.ranef <- function(y, theta, cens, X1, X2, n, limit, type, squared) {
  a <- exp(theta[1])
  beta1 <- theta[2:(2+ncol(X1)-1)]
  beta2 <- theta[(2+ncol(X1)):(2+ncol(X1)+ncol(X2)-1)]
  sig.block <- exp(theta[length(theta)])
  approx <- int.approx(integrand.ranef, yi=y, a=a, beta1=beta1, beta2=beta2, sig.block=sig.block, cens=cens, X1=X1, X2=X2, n=n, limit=limit, type=type, squared=squared)
  return(approx)
}

ranef.ew.mix <- function(theta, y, cens, X1, X2, n, limit, type, squared=FALSE) {
  if(missing(n)) n <- 10
  if(missing(limit)) limit <- 30
  if(missing(type)) type <- "GQ"
  yi <- split(y, spod$day)
  censi <- split(cens, spod$day)
  X1i <- split(X1, spod$day)
  for(i in 1:3) X1i[[i]] <- matrix(X1i[[i]], ncol=ncol(X1))
  X2i <- split(X2, spod$day)
  for(i in 1:3) X2i[[i]] <- matrix(X2i[[i]], ncol=ncol(X2))
  ranefs <- NULL
  for(i in 1:3) {
    ranefs[i] <- integral.ranef(y=yi[[i]], theta=theta, cens=censi[[i]], X1=X1i[[i]], X2=X2i[[i]], n=n, limit=limit, type=type, squared=squared)
  }
  return(ranefs)
}